local csv ={}
csv.version="1.0"

function csv.trim(s)
  return (s:gsub("^%s*(.-)%s*$", "%1"))
end

-- Used to escape "'s by toCSV
function csv.escape (s)
  if string.find(s, '[,"]') then
    s = '"' .. string.gsub(s, '"', '""') .. '"'
  end
  return s
end

-- Convert from table to CSV string
-- inputs are a table with values and a separator character
function csv.to (t, s)
  local str = ""
  for _,p in pairs(t) do
    str = str .. s .. escapeCSV(p)
  end
  return string.sub(str, 2)      -- remove first comma
end

function csv.saveDistrib (fn, d)
    fh = io.open (fn, "a+")

    fh:write("x, y","\n")
    for k, v in ipairs(d) do
        fh:write(k, ", ", v,"\n")
    end
    fh:close()
end

--[[
    Save a histogram as a CSV file
    -- histogram must be a table with three field per entry
    -- v.min, v.max, v.num
    -- inputs are a histogram and a file name
]]
function csv.saveHist (fn, hist)
    assert (fn ~= nil)
    fh = io.open (fn, "a+")

    fh:write("min; max; value","\n")
    for k, v in ipairs(hist) do
        assert (v.min ~= nil and v.max ~= nil and v.num ~= nil)
        fh:write(v.min, "; ", v.max,"; ", v.num,"\n")
    end
    fh:close()
end

--- write a CSV file from a table 
-- 

function csv.toFile (fhandle, tab, sep)
	for i=1,#tab do
		str = toCSV (tab[i],sep)
		str = str .. "\n"
		fhandle:write(str)
	end
end

function csv.parseLine (line,sep) 
	local res = {}
	local pos = 1
	sep = sep or ','
	while true do 
		local c = string.sub(line,pos,pos)
		if (c == "") then break end
		if (c == '"') then
			-- quoted value (ignore separator within)
			local txt = ""
			repeat
				local startp,endp = string.find(line,'^%b""',pos)
				txt = txt..string.sub(line,startp+1,endp-1)
				pos = endp + 1
				c = string.sub(line,pos,pos) 
				if (c == '"') then txt = txt..'"' end 
				-- check first char AFTER quoted string, if it is another
				-- quoted string without separator, then append it
				-- this is the way to "escape" the quote char in a quote. example:
				--   value1,"blub""blip""boing",value3  will result in blub"blip"boing  for the middle
			until (c ~= '"')
			table.insert(res,txt)
			assert(c == sep or c == "")
			pos = pos + 1
		else	
			-- no quotes used, just look for the first separator
			local startp,endp = string.find(line,sep,pos)
			if (startp) then 
				table.insert(res,string.sub(line,pos,startp-1))
				pos = endp + 1
			else
				-- no separator found -> use rest of string and terminate
				table.insert(res,string.sub(line,pos))
				break
			end 
		end
	end
	return res
end

return csv