--[[
	Patterns in Life Game

]]
-- for more pattern, look at the "oscillators" and the "spaceships"
-- in http://www.conwaylife.com/wiki/Main_Page

local life = {}

function life.insert_pattern (cs1, cs2, x0, y0)
	for i = 0, (cs2.ydim - 1) do
		for j = 0, (cs2.xdim - 1) do
			cs1:getCell(Coord{x = x0 + j, y = y0 + i}).state = 
                                        cs2:getCell(Coord{x = j, y = i}).state
		end
	end
end 

function life.brian_osc()
	local cs = CellularSpace{ 
		xdim = 4, 
		ydim = 4
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=1,y=1}).state = ALIVE
	cs:getCell(Coord{x=1,y=2}).state = ALIVE
	cs:getCell(Coord{x=2,y=1}).state = ALIVE
	cs:getCell(Coord{x=2,y=2}).state = ALIVE
	cs:getCell(Coord{x=1,y=0}).state = DYING
	cs:getCell(Coord{x=0,y=2}).state = DYING
	cs:getCell(Coord{x=2,y=3}).state = DYING
	cs:getCell(Coord{x=3,y=1}).state = DYING

return cs
end



function life.glider()
	local cs = CellularSpace{ 
		xdim = 3, 
		ydim = 3
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=1,y=0}).state = ALIVE
	cs:getCell(Coord{x=2,y=1}).state = ALIVE
	cs:getCell(Coord{x=0,y=2}).state = ALIVE
	cs:getCell(Coord{x=1,y=2}).state = ALIVE
	cs:getCell(Coord{x=2,y=2}).state = ALIVE

return cs
end

--[[  
Pulsar oscillator in Life 

..OOO...OOO

O....O.O....O
O....O.O....O
O....O.O....O
..OOO...OOO

..OOO...OOO
O....O.O....O
O....O.O....O
O....O.O....O

..OOO...OOO
]]
function life.pulsar()
	local cs = CellularSpace{ 
		xdim = 13, 
		ydim = 13
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=2,y=0}).state = ALIVE
	cs:getCell(Coord{x=3,y=0}).state = ALIVE
	cs:getCell(Coord{x=4,y=0}).state = ALIVE

	cs:getCell(Coord{x=8,y=0}).state = ALIVE
	cs:getCell(Coord{x=9,y=0}).state = ALIVE
	cs:getCell(Coord{x=10,y=0}).state = ALIVE

	cs:getCell(Coord{x=0,y=2}).state = ALIVE
	cs:getCell(Coord{x=5,y=2}).state = ALIVE
	cs:getCell(Coord{x=7,y=2}).state = ALIVE
	cs:getCell(Coord{x=12,y=2}).state = ALIVE

	cs:getCell(Coord{x=0,y=3}).state = ALIVE
	cs:getCell(Coord{x=5,y=3}).state = ALIVE
	cs:getCell(Coord{x=7,y=3}).state = ALIVE
	cs:getCell(Coord{x=12,y=3}).state = ALIVE

	cs:getCell(Coord{x=0,y=4}).state = ALIVE
	cs:getCell(Coord{x=5,y=4}).state = ALIVE
	cs:getCell(Coord{x=7,y=4}).state = ALIVE
	cs:getCell(Coord{x=12,y=4}).state = ALIVE

	cs:getCell(Coord{x=2,y=5}).state = ALIVE
	cs:getCell(Coord{x=3,y=5}).state = ALIVE
	cs:getCell(Coord{x=4,y=5}).state = ALIVE

	cs:getCell(Coord{x=8,y=5}).state = ALIVE
	cs:getCell(Coord{x=9,y=5}).state = ALIVE
	cs:getCell(Coord{x=10,y=5}).state = ALIVE

	cs:getCell(Coord{x=2,y=7}).state = ALIVE
	cs:getCell(Coord{x=3,y=7}).state = ALIVE
	cs:getCell(Coord{x=4,y=7}).state = ALIVE

	cs:getCell(Coord{x=8,y=7}).state = ALIVE
	cs:getCell(Coord{x=9,y=7}).state = ALIVE
	cs:getCell(Coord{x=10,y=7}).state = ALIVE

	cs:getCell(Coord{x=0,y=8}).state = ALIVE
	cs:getCell(Coord{x=5,y=8}).state = ALIVE
	cs:getCell(Coord{x=7,y=8}).state = ALIVE
	cs:getCell(Coord{x=12,y=8}).state = ALIVE

	cs:getCell(Coord{x=0,y=9}).state = ALIVE
	cs:getCell(Coord{x=5,y=9}).state = ALIVE
	cs:getCell(Coord{x=7,y=9}).state = ALIVE
	cs:getCell(Coord{x=12,y=9}).state = ALIVE

	cs:getCell(Coord{x=0,y=10}).state = ALIVE
	cs:getCell(Coord{x=5,y=10}).state = ALIVE
	cs:getCell(Coord{x=7,y=10}).state = ALIVE
	cs:getCell(Coord{x=12,y=10}).state = ALIVE

	cs:getCell(Coord{x=2,y=12}).state = ALIVE
	cs:getCell(Coord{x=3,y=12}).state = ALIVE
	cs:getCell(Coord{x=4,y=12}).state = ALIVE

	cs:getCell(Coord{x=8,y=12}).state = ALIVE
	cs:getCell(Coord{x=9,y=12}).state = ALIVE
	cs:getCell(Coord{x=10,y=12}).state = ALIVE

return cs
end

--[[ Figure-eight oscillator in Life 

OO
OO.O
....O
.O
..O.OO
....OO
]]

function life.figureeight()
	local cs = CellularSpace{ 
		xdim = 6, 
		ydim = 6
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=0,y=0}).state = ALIVE
	cs:getCell(Coord{x=1,y=0}).state = ALIVE

	cs:getCell(Coord{x=0,y=1}).state = ALIVE
	cs:getCell(Coord{x=1,y=1}).state = ALIVE
	cs:getCell(Coord{x=3,y=1}).state = ALIVE

	cs:getCell(Coord{x=4,y=2}).state = ALIVE

	cs:getCell(Coord{x=1,y=3}).state = ALIVE

	cs:getCell(Coord{x=2,y=4}).state = ALIVE
	cs:getCell(Coord{x=4,y=4}).state = ALIVE
	cs:getCell(Coord{x=5,y=4}).state = ALIVE

	cs:getCell(Coord{x=4,y=5}).state = ALIVE
	cs:getCell(Coord{x=5,y=5}).state = ALIVE


return cs
end
--[[ 
...OO
..O..O
.O....O
O......O
O......O
.O....O
..O..O
...OO

]]

function life.octagon()
	local cs = CellularSpace{ 
		xdim = 8, 
		ydim = 8
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=3,y=0}).state = ALIVE
	cs:getCell(Coord{x=4,y=0}).state = ALIVE

	cs:getCell(Coord{x=2,y=1}).state = ALIVE
	cs:getCell(Coord{x=5,y=1}).state = ALIVE

	cs:getCell(Coord{x=1,y=2}).state = ALIVE
	cs:getCell(Coord{x=6,y=2}).state = ALIVE

	cs:getCell(Coord{x=0,y=3}).state = ALIVE
	cs:getCell(Coord{x=7,y=3}).state = ALIVE

	cs:getCell(Coord{x=0,y=4}).state = ALIVE
	cs:getCell(Coord{x=7,y=4}).state = ALIVE

	cs:getCell(Coord{x=1,y=5}).state = ALIVE
	cs:getCell(Coord{x=6,y=5}).state = ALIVE

	cs:getCell(Coord{x=2,y=6}).state = ALIVE
	cs:getCell(Coord{x=5,y=6}).state = ALIVE
	
	cs:getCell(Coord{x=3,y=7}).state = ALIVE
	cs:getCell(Coord{x=4,y=7}).state = ALIVE

return cs
end

--[[ 
..O....O
OO.OOOO.OO
..O....O
]]

function life.pentadecathlon()

	local cs = CellularSpace{ 
		xdim = 10, 
		ydim = 3
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=2,y=0}).state = ALIVE
	cs:getCell(Coord{x=7,y=0}).state = ALIVE

	cs:getCell(Coord{x=0,y=1}).state = ALIVE
	cs:getCell(Coord{x=1,y=1}).state = ALIVE
	cs:getCell(Coord{x=3,y=1}).state = ALIVE
	cs:getCell(Coord{x=4,y=1}).state = ALIVE
	cs:getCell(Coord{x=5,y=1}).state = ALIVE
	cs:getCell(Coord{x=6,y=1}).state = ALIVE
	cs:getCell(Coord{x=8,y=1}).state = ALIVE
	cs:getCell(Coord{x=9,y=1}).state = ALIVE

	cs:getCell(Coord{x=2,y=2}).state = ALIVE
	cs:getCell(Coord{x=7,y=2}).state = ALIVE
return cs
end

--[[
.OO
OO
.O
]]

function life.rpentomino()
	local cs = CellularSpace{ 
		xdim = 3, 
		ydim = 3
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=1,y=0}).state = ALIVE
	cs:getCell(Coord{x=2,y=0}).state = ALIVE
	cs:getCell(Coord{x=0,y=1}).state = ALIVE
	cs:getCell(Coord{x=1,y=1}).state = ALIVE
	cs:getCell(Coord{x=1,y=2}).state = ALIVE

return cs
end

--[[
 0123456789012
 .0...........
1.000.......00
2....0......0.
3...00....0.0.
4.........00..
5.............
6.....000.....
7.....000.....
8..00.........
9.0.0....00...
 .0......0....
100.......000.
2...........0.
]]

function life.dinnertable()
	local cs = CellularSpace{ 
		xdim = 13, 
		ydim = 13
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=1,y=0}).state = ALIVE
	cs:getCell(Coord{x=1,y=1}).state = ALIVE
	cs:getCell(Coord{x=2,y=1}).state = ALIVE
	cs:getCell(Coord{x=3,y=1}).state = ALIVE
	cs:getCell(Coord{x=11,y=1}).state = ALIVE
	cs:getCell(Coord{x=12,y=1}).state = ALIVE
	cs:getCell(Coord{x=4,y=2}).state = ALIVE
	cs:getCell(Coord{x=11,y=2}).state = ALIVE
	cs:getCell(Coord{x=3,y=3}).state = ALIVE
	cs:getCell(Coord{x=4,y=3}).state = ALIVE
	cs:getCell(Coord{x=9,y=3}).state = ALIVE
	cs:getCell(Coord{x=11,y=3}).state = ALIVE
	cs:getCell(Coord{x=9,y=4}).state = ALIVE
	cs:getCell(Coord{x=10,y=4}).state = ALIVE
	cs:getCell(Coord{x=5,y=6}).state = ALIVE
	cs:getCell(Coord{x=6,y=6}).state = ALIVE
	cs:getCell(Coord{x=7,y=6}).state = ALIVE
	cs:getCell(Coord{x=5,y=7}).state = ALIVE
	cs:getCell(Coord{x=6,y=7}).state = ALIVE
	cs:getCell(Coord{x=7,y=7}).state = ALIVE
	cs:getCell(Coord{x=2,y=8}).state = ALIVE
	cs:getCell(Coord{x=3,y=8}).state = ALIVE
	cs:getCell(Coord{x=1,y=9}).state = ALIVE
	cs:getCell(Coord{x=3,y=9}).state = ALIVE
	cs:getCell(Coord{x=8,y=9}).state = ALIVE
	cs:getCell(Coord{x=9,y=9}).state = ALIVE
	cs:getCell(Coord{x=1,y=10}).state = ALIVE
	cs:getCell(Coord{x=8,y=10}).state = ALIVE
	cs:getCell(Coord{x=0,y=11}).state = ALIVE
	cs:getCell(Coord{x=1,y=11}).state = ALIVE
	cs:getCell(Coord{x=9,y=11}).state = ALIVE
	cs:getCell(Coord{x=10,y=11}).state = ALIVE
	cs:getCell(Coord{x=11,y=11}).state = ALIVE
	cs:getCell(Coord{x=11,y=12}).state = ALIVE

return cs
end

--[[
 0123456
 ...oo..
1.o....0
2o......
3o.....0
4oooooo.
]]

function life.heavyspaceship()
	local cs = CellularSpace{ 
		xdim = 7, 
		ydim = 7
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=3,y=0}).state = ALIVE
	cs:getCell(Coord{x=4,y=0}).state = ALIVE
	cs:getCell(Coord{x=1,y=1}).state = ALIVE
	cs:getCell(Coord{x=6,y=1}).state = ALIVE
	cs:getCell(Coord{x=0,y=2}).state = ALIVE
	cs:getCell(Coord{x=0,y=3}).state = ALIVE
	cs:getCell(Coord{x=6,y=3}).state = ALIVE
	cs:getCell(Coord{x=0,y=4}).state = ALIVE
	cs:getCell(Coord{x=1,y=4}).state = ALIVE
	cs:getCell(Coord{x=2,y=4}).state = ALIVE
	cs:getCell(Coord{x=3,y=4}).state = ALIVE
	cs:getCell(Coord{x=4,y=4}).state = ALIVE
	cs:getCell(Coord{x=5,y=4}).state = ALIVE

return cs
end

function life.rabbits()
	local cs = CellularSpace{ 
		xdim = 7, 
		ydim = 7
	}
	forEachCell(cs, function(cell)
		cell.state = DEAD
	end)

	cs:getCell(Coord{x=0,y=0}).state = ALIVE
	cs:getCell(Coord{x=0,y=4}).state = ALIVE
	cs:getCell(Coord{x=0,y=5}).state = ALIVE
	cs:getCell(Coord{x=0,y=6}).state = ALIVE
	cs:getCell(Coord{x=1,y=0}).state = ALIVE
	cs:getCell(Coord{x=1,y=1}).state = ALIVE
	cs:getCell(Coord{x=1,y=2}).state = ALIVE
	cs:getCell(Coord{x=1,y=5}).state = ALIVE
	cs:getCell(Coord{x=2,y=1}).state = ALIVE

return cs
end

return life