--[[ 
Utility functions for lua

]]

local util ={}
util.version="1.0"

--[[
	Wait some time before the next iteration 
	to help visualize the model better
]]
function util.wait(n)
	for i = 0, n*100000 do io.write("") end -- do nothing 100.000 times
end
--[[
    Clone values of a lua table
]]-- 
function util.clone (luaTable)

	local newTable = {}
	for key,val in pairs(luaTable) do
		if type(val) == 'table' then
			val = util.clone (val)
		end
		newTable [key] = val
	end
	return newTable
end

--[[
  print values of a lua Table
]]
function util.print (t)
    for k,v in pairs(t) do
        if type(v) == 'table' then
            util.print (v)
        end
        print (k,v)
    end
end


function util.trace(event, line)
    local s = debug.getinfo(2).short_src
    print(s .. ": " .. line)
	io.flush()
end



function util.dump(o)
    if type(o) == 'table' then
        local s = '{ '
        for k,v in pairs(o) do
                if type(k) ~= 'number' then k = '"'..k..'"' end
                s = s .. '['..k..'] = ' .. util.dump(v) .. ','
        end
        return s .. '} '
    else
        return tostring(o)
    end
end

-------------------------------------------------------------------------------
--- Método de depuração imprime "qualquer" tipo de estrutura
-- de dados em Lua
--
-- @release Copiado do livro Lua Programming Gems pág: 31
-- 
-- @param value valor
-- @param depth profundidade
-- @param key chave
strdump = ""
function util.vardump(value, depth, key)
	local linePrefix = ""
	local spaces = ""
	
	if (key ~= nil) then
		linePrefix = "["..key.."] = "
	end
	
	if (depth == nil) then
		depth = 0
	else
		depth = depth + 1
		for i = 1, depth do
			spaces = spaces .. "  "
		end
	end
	
	if (type(value) == 'table') then
		mTable = getmetatable(value)
		if (mTable == nil) then
			print(spaces .. linePrefix.. "(table) ")
			strdump = strdump .. spaces .. linePrefix.. "(table)\n"
		else 
			print(spaces.. "(metatable) ")
			value = mTable
			strdump = strdump .. spaces.. "(metatable)\n"
		end
		for tableKey, tableValue in pairs(value) do
			util.vardump(tableValue, depth, tableKey)
		end
	elseif ((type(value) == 'function') or (type(value) == 'thread')
		or (type(value) == 'userdata') or (type(value) == nil))	then
		print(spaces ..tostring(value))
		strdump = strdump .. spaces ..tostring(value) .."\n"
	else 
		print(spaces..linePrefix.."("..type(value)..") "..tostring(value))
		strdump = strdump .. spaces..linePrefix.."("..type(value)..") "..tostring(value).."\n"
	end
	io.flush()
	return strdump
end

return util